C	THIS IS A COLLECTION OF THE FOLLOWING SUBROUTINES
C       AND FUNCTIONS;

C           MODIFIED BY HALDUN O. TEZEL 1986 


C             	WASIN1
C		WASAP1
C		WASTD1 -----> function
C		WASTL1 -----> function


	SUBROUTINE WASIN1 (VI1,VI2,VO1,VO2,L)

C  THIS SUBROUTINE PROVIDES INITIAL ESTIMATES OF TEMPERATURE
C  AND SPECIFIC VOLUME AS FUNCTIONS OF VI1 AND VI2
C
C  PARAMETERS:
C 	L	Parameter to be used only if the independent 
C		variables are Pressure and Temperature.
C		Then if;
C		L=100 ----> Vold calculated from subcooled lig.
C		L=200 ----> Vold calculated from superheated vap.
C		corresponding to VI1 and VI2.
C		
C	TOLD	Temperature, Deg.C
C	VOLD	Specific volume, m3/Kg
C	VI1	First independent variable
C	VI2	Second independent variable
C
C  LOCAL VARIABLES:
C	POLD	Last value of pressure, Pa
C	PSATF	Saturation pressure at temperature TOLD, Pa
C	PSL	Pressure at water region at TOLD, Pa
C	PX	An arbitrary pressure close to critical press.
C		and set to 2E7, Pa
C
	IMPLICIT REAL*8 (A-H,O-Z)
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER

	COMMON /WASPR5/ F1,F2,F11,F12,F22,F21,TES1,TES2,TNEW,VNEW,TOLD,
     *                  VOLD,POLD,IS2


C  THE FOLLOWING TWO ARE FUNCTIONS TO BE INSERTED 

C  	$INCLUDE:'WASTD1.FOR'
C	$INCLUDE:'WASTL1.FOR'

	WRITE (*,5)
 5	FORMAT (10X,'WASIN1',/)

	DATA AL0,AL1,AL2 /1.574373327D1,-3.417061978D1,1.931380707D1/

	TOLD=VO1

C  CALCULATE AN INITIAL SPECIFIC VOLUME
	IF (L.EQ.100) GOTO 46
	IF (L.EQ.200) GOTO 45
	T=(TOLD+ZZZ273)/647.3
	PSATF=0.0
	PSL=0.0
	IF (T.LE.1.0)  CALL WASAP1 (PSATF,TOLD,QUO)
	IF (T.GT.0.96269) PSL=(AL0+T*(AL1+T*AL2))
	IF (POLD.GT.PSL .AND. T.GT.0.96269) GOTO 40
	IF (T.LT.0.96269) GOTO 44
 45	CONTINUE
	PX=2.0D7
	IF (POLD.GT.PX .AND. TOLD.LT.374.15) GOTO 50
	VO2=WASTD1(POLD,TOLD)
	GOTO 99
 40 	CONTINUE
	IF (T.GT.1.001) GOTO 42
	IF (POLD.GT.PSATF) GOTO 43
 44	CONTINUE
	IF (POLD.LE.PSATF) GOTO 45
 46	CONTINUE
	VO2=WASTL1(POLD,TOLD)
	GOTO 99
 42 	CONTINUE
	VO2=WASTD1(PSL,TOLD)*PSL/POLD
	GOTO 99
 43	CONTINUE
	VO2=0.7336218
	GOTO 99
 50	CONTINUE
	VO2=(4.6515D-3)*(2.137D7/POLD)**3
 99	RETURN
	END


	SUBROUTINE WASAP1 (VAR1,VAR2,QUO)
C
C  SATURATION LINE EQUATION. SOURCE: 1968 IFC FORMULATION 
C
	IMPLICIT REAL*8 (A-H,O-Z)
	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER
	
	DIMENSION A(5),AK(12),TU(12),DTU(12)

	DATA A /1.0002D0,1.3464D-1,1.49919D-2,1.01085D-3,2.9045D-5/

	DATA AK /-4.0596821D0,5.1322555D0,-1.1842407D0,1.1779592D-1,
     *	-5.157642D-3,-1.4689537D-3,5.3622818D-4,1.2455399D-4,
     *	-4.9154288D-5,4.6302565D-5,1.5301334D-5,-2.095453D-5/

	DATA XK1,XK2,XK3 /9.5D-1,1.45220717D0,-8.4878953D-1/

	WRITE (*,5)
 5	FORMAT (10X,'WASAP1',/)

	ITEMPS=0
	IF (VAR2.GT.ZZZM2 .OR. VAR2.LT.374.15D0) GOTO 51
	IF (VAR2.LE.ZZZM2) VAR1=611.3D0
	IF (VAR2.GE.374.15D0) VAR1=220.9D5
	ITERM=5
C	KERLV=5                    THESE ARE ERROR MESSAGES
C	ERRMES(1)=SNAME
C	ERRMES(2)=VAR2
C	CALL TERMN1

	GOTO 99
 51	CONTINUE
	THETA=(VAR2+ZZZ273)/647.3D0
	U=(ZZZ2*(ZZZ1/THETA-XK1)**0.4D0-XK2)/XK3
	TU(1)=ZZZ1
	TU(2)=U

	DO 10 I=3,12
	J=I-1
	K=I-2
 10	TU(I)=ZZZ2*U*TU(J)-TU(K)
	SUM=ZZZ0
	DO 20 I=1,12
 20	SUM=SUM+AK(I)*TU(I)
	BETAK=DEXP(SUM)
	PSF=BETAK*2.212D7
	DU=-ZZZ1*0.4D0*ZZZ2/(XK3*(ZZZ1/THETA-XK1)**0.6D0*THETA**2)
	DTU(1)=ZZZ0
	DTU(2)=DU
	DO 30 I=3,12
	J=I-1
	K=I-2
 30	DTU(I)=ZZZ2*(DU*TU(J)+U*DTU(J))-DTU(K)
	DSUM=ZZZ0
	DO 40 I=1,12
 40	DSUM=DSUM+AK(I)*DTU(I)
	DBETAK=BETAK*DSUM
	QUO=DBETAK*2.212D7/647.3D0
	IF (ITEMPS.EQ.0) GOTO 45
	GOTO 12

C	ENTRY WASATI (VAR1,VAR2,QUO)
	IF (VAR1.GT.611.3D0 .OR. VAR1.LT.220.9D5) GOTO 3
C	ITERM=5                  ERROR MESSAGES
C	KERLV=5
C	ERRMES(1)=SNAM2
C	ERRMES(2)=VAR1
	IF (VAR1.LT.611.3D0) VAR2=ZZZM2
	IF (VAR1.GE.220.9D5) VAR2=374.15D0
C	CALL TERMN1              ERROR MESSAGE SUBROUTINE
	GOTO 99
 3	ITEMPS=1
	NDEG=4
	SUM=ZZZ0
	FUN=DLOG(VAR1/2.212D7)
	DO 1 I=1,NDEG
	J=NDEG+2-I
	SUM=FUN*(A(J)+SUM)
 1	CONTINUE
	THETA=A(I)+SUM
	VAR2=THETA*647.3D0-ZZZ273
	I=1
	GOTO 51
 12	CONTINUE
	ZDELTE=(PSF-VAR1)/QUO
	VAR2=VAR2-ZDELTE
	DIF=DABS((VAR1-PSF)*ZZZE2/VAR1)
	PTEST=1.0D-5
	IF (DIF.LE.PTEST) GOTO 99
	I=I+1
	IF (I.GE.20) GOTO 31
	GOTO 51
 31 	CONTINUE
	WRITE(6,100) ZZZAST,I,PSF
 45	CONTINUE
	VAR1=PSF
 100	FORMAT(A8,'    WASATI ITERATIONS ',I4,'   PSF= ',D15.6//)
 99	RETURN
	END


	FUNCTION WASTD1 (PSF,TCENT)

C  DERIVATIVE OF THE B-FUNCTION WRT PRESSURE.  THE FUNCTION RETURNS
C  THE SPECIFIC VOLUME OF VAPOUR
C  SOURCE:  IFC FORMULATION FOR INDUSTRIAL AND SCIENTIFIC USE.
C  SECTION 6.2.

	IMPLICIT REAL*8 (A-H,O-Z)

	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER

       DATA AL0,AL1,AL2 /1.574373327D01,-3.417061978D01,1.931380707D01/

	DATA XI1,SMALLB /4.260321148D0,0.76333333333D0/

      	DATA B11,B12,B21,B22,B23,B31,B32,B41,B42,B51,B52,B53,B61,B62,
     *	B71,B72,B81,B82,B90,B91,B92,B93,B94,B95,B96 /
     *  6.670375918D-2,1.388983801D0,8.390104328D-2,2.614670893D-2,
     *	-3.373439453D-2,4.520918904D-1,1.069036614D-1,-5.975336707D-1,
     *  -8.847535804D-2,5.958051609D-1,-5.159303373D-1,2.075021122D-1,
     *	1.190610271D-1,-9.867174132D-2,1.683998803D-1,-5.809438001D-2,
     *	6.522390126D-3,5.710218649D-4,1.936587558D2,-1.388522425D3,
     *	4.126607219D3,-6.508211677D3,5.745984054D3,-2.693088365D3,
     *	5.235718623D2 /

	DATA BB61,BB71,BB81,BB82
     *	/4.006073948D-1,8.636081627D-2,-8.532322921D-1,3.460208861D-1/

	WRITE (*,5)
 5	FORMAT (10X,'WASTD1',/)

	THETA=(TCENT+ZZZ273)/647.3
	BETAK=PSF/2.212D7
	QUANT=ZZZ1-THETA
	BETAL=AL0+THETA*(AL1+THETA*AL2)
	BIGX=DEXP(SMALLB*QUANT)
	TERM1=XI1*THETA/BETAK
	SA1=B11*BIGX**13+B12*BIGX**3
	SA2=B21*BIGX**18+B22*BIGX**2+B23*BIGX
	SA3=B31*BIGX**18+B32*BIGX**10
	SA4=B41*BIGX**25+B42*BIGX**14
	SA5=B51*BIGX**32+B52*BIGX**28+B53*BIGX**24
	TERM2=SA1+BETAK*(ZZZ2*SA2+BETAK*(ZZZ3*SA3+BETAK*(ZZZ4*SA4+
     *	BETAK*ZZZ5*SA5)))
	DEN=(ZZZ1/BETAK**4+BB61*BIGX**14)**2
	TOP=B61*BIGX**12+B62*BIGX**11
	PART1=TOP*ZZZ4/(DEN*BETAK**5)
	DEN=(ZZZ1/BETAK**5+BB71*BIGX**19)**2
	TOP=B71*BIGX**24+B72*BIGX**18
	PART2=TOP*ZZZ5/(DEN*BETAK**6)
	DEN=(ZZZ1/BETAK**6+BB81*BIGX**54+BB82*BIGX**27)**2
	TOP=B81*BIGX**24+B82*BIGX**14
	PART3=TOP*ZZZ6/(DEN*BETAK**7)
	TERM3=PART1+PART2+PART3
	SA1=B90+B91*BIGX+B92*BIGX**2+B93*BIGX**3+B94*BIGX**4+
     *      B95*BIGX**5+B96*BIGX**6
	TERM4=11.0D0*SA1*(BETAK/BETAL)**10
	DZDP=TERM1-TERM2-TERM3+TERM4
	
	WASTD1=DZDP*3.17D-3

 99	RETURN
	END


	FUNCTION WASTL1 (PSF,TCENT)

C SPECIFIC VOLUME OF WATER AS A FUNCTION OF PRESSURE AND TEMPERATURE
C SOURCE: 1968 IFC FORMULATION FOR SCIENTIFIC AND GENERAL USE.
C Section 6.1 The A-Function

	IMPLICIT REAL*8 (A-H,O-Z)

	COMMON /CONST5/ ZZZ0,ZZZ1,ZZZ2,ZZZ3,ZZZ4,ZZZ5,ZZZ6,ZZZ7,ZZZ8,
     *	ZZZ9,ZZZE2,ZZZE3,ZZZE4,ZZZE5,ZZZE6,ZZZM2,ZZZM3,ZZZM4,ZZZM5,
     *	ZZZM6,ZZZ01,ZZZ10,ZZZ273,ZZZ314,ZZZ981,ZZZER

	DATA A11,A12,A13,A14,A15,A16,A17,A18,A19,A20,A21,A22 /
     *	7.982692717D0,-2.616571843D-2,1.52241179D-3,2.284279054D-2,
     *	2.421647003D2,1.269716088D-10,2.074838328D-7,2.174020353D-8,
     *	1.105710498D-9,1.293441934D1,1.308119072D-5,6.047626338D-14 /

	DATA S1,S2,S3,S4,S5,S6,S7,S8,S9,S10,S11,S12 /
     *	8.438375405D-1,5.362162162D-4,1.72D0,7.342278489D-2,
     *	4.97585887D-2,6.537154300D-1,1.15D-6,1.5108D-5,
     *  1.4188D-1,7.002753165D0,2.995284926D-4,2.04D-1 /

	WRITE (*,7)
 7	FORMAT (10X,'WASTL1',/)

	T=(TCENT+ZZZ273)/647.3
	BETAK=PSF/2.212D7
	Y=ZZZ1-S1*T**2-S2/T**6
	SUM4=S3*Y**2-ZZZ2*S4*T+ZZZ2*S5*BETAK
	IF (SUM4.LT.ZZZ0) GOTO 5
	Z=Y+DSQRT(SUM4)
	GOTO 6
 5	CONTINUE
	Z=Y
 6	CONTINUE
	POWER=ZZZ5/17.0

C   CALCULATE THE GIBBS FREE ENERGY DERIVATIVE WRT PRESSURE

	TERM1=A11*S5/Z**POWER
	TERM2=A12+A13*T+A14*T**2+A15*(S6-T)**10+A16/(S7+T**19)
	TERM3=(A17+ZZZ2*A18*BETAK+ZZZ3*A19*BETAK**2)/(S8+T**11)
	TERM4=(S11-ZZZ3/(S10+BETAK)**4)*A20*T**18*(S9+T**2)
	TERM5=ZZZ3*A21*(S12-T)*BETAK**2
	TERM6=ZZZ4*A22*BETAK**3/T**20 
	DZDB=TERM1+TERM2-TERM3-TERM4+TERM5+TERM6
	WASTL1=DZDB*3.170D-3
 99	RETURN
	END
